package io.gitlab.jfronny.inceptum.launcher.api.account;

import io.gitlab.jfronny.commons.HttpUtils;
import io.gitlab.jfronny.inceptum.launcher.model.microsoft.*;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.*;

public class MicrosoftAuthAPI {
    public static final String MICROSOFT_LOGIN_CLIENT_ID = "90890812-00d1-48a8-8d3f-38465ef43b58";
    public static final int MICROSOFT_LOGIN_REDIRECT_PORT = 28562;
    public static final String MICROSOFT_LOGIN_REDIRECT_URL = "http://127.0.0.1:" + MICROSOFT_LOGIN_REDIRECT_PORT;
    public static final String MICROSOFT_LOGIN_REDIRECT_URL_ENCODED = "http%3A%2F%2F127.0.0.1%3A"
            + MICROSOFT_LOGIN_REDIRECT_PORT;
    public static final String[] MICROSOFT_LOGIN_SCOPES = {"XboxLive.signin", "XboxLive.offline_access"};

    // General Microsoft login constants
    public static final String MICROSOFT_LOGIN_URL = "https://login.live.com/oauth20_authorize.srf" + "?client_id="
            + MICROSOFT_LOGIN_CLIENT_ID
            + "&prompt=select_account&cobrandid=8058f65d-ce06-4c30-9559-473c9275a65d&response_type=code" + "&scope="
            + String.join("%20", MICROSOFT_LOGIN_SCOPES) + "&redirect_uri=" + MICROSOFT_LOGIN_REDIRECT_URL_ENCODED;
    public static final String MICROSOFT_AUTH_TOKEN_URL = "https://login.live.com/oauth20_token.srf";
    public static final String MICROSOFT_XBL_AUTH_TOKEN_URL = "https://user.auth.xboxlive.com/user/authenticate";
    public static final String MICROSOFT_XSTS_AUTH_TOKEN_URL = "https://xsts.auth.xboxlive.com/xsts/authorize";
    public static final String MICROSOFT_MINECRAFT_LOGIN_URL = "https://api.minecraftservices.com/launcher/login";
    public static final String MICROSOFT_MINECRAFT_PROFILE_URL = "https://api.minecraftservices.com/minecraft/profile";
    public static final String MICROSOFT_MINECRAFT_ENTITLEMENTS_URL = "https://api.minecraftservices.com/entitlements/license?requestId=";

    public static OauthTokenResponse tradeCodeForAccessToken(String code) throws IOException, URISyntaxException {
        return HttpUtils.post(MICROSOFT_AUTH_TOKEN_URL)
                .bodyForm(Map.of("client_id", MICROSOFT_LOGIN_CLIENT_ID,
                        "code", code,
                        "grant_type", "authorization_code",
                        "redirect_uri", MICROSOFT_LOGIN_REDIRECT_URL,
                        "scope", String.join(" ", MICROSOFT_LOGIN_SCOPES)))
                .sendSerialized(OauthTokenResponse.class);
    }

    public static OauthTokenResponse refreshAccessToken(String refreshToken) throws IOException, URISyntaxException {
        return HttpUtils.post(MICROSOFT_AUTH_TOKEN_URL)
                .bodyForm(Map.of("client_id", MICROSOFT_LOGIN_CLIENT_ID,
                        "refresh_token", refreshToken,
                        "grant_type", "refresh_token",
                        "redirect_uri", MICROSOFT_LOGIN_REDIRECT_URL))
                .sendSerialized(OauthTokenResponse.class);
    }

    public static XboxLiveAuthResponse getXBLToken(String accessToken) throws IOException, URISyntaxException {
        Map<Object, Object> properties = new HashMap<>();
        properties.put("AuthMethod", "RPS");
        properties.put("SiteName", "user.auth.xboxlive.com");
        properties.put("RpsTicket", "d=" + accessToken);

        Map<Object, Object> data = new HashMap<>();
        data.put("Properties", properties);
        data.put("RelyingParty", "http://auth.xboxlive.com");
        data.put("TokenType", "JWT");

        return HttpUtils.post(MICROSOFT_XBL_AUTH_TOKEN_URL)
                .header("x-xbl-contract-version", "1")
                .bodySerialized(data)
                .sendSerialized(XboxLiveAuthResponse.class);
    }

    public static XboxLiveAuthResponse getXstsToken(String xblToken) throws IOException, URISyntaxException {
        Map<Object, Object> properties = new HashMap<>();
        properties.put("SandboxId", "RETAIL");

        List<String> userTokens = new ArrayList<>();
        userTokens.add(xblToken);
        properties.put("UserTokens", userTokens);

        Map<Object, Object> data = new HashMap<>();
        data.put("Properties", properties);
        data.put("RelyingParty", "rp://api.minecraftservices.com/");
        data.put("TokenType", "JWT");

        return HttpUtils.post(MICROSOFT_XSTS_AUTH_TOKEN_URL)
                .header("x-xbl-contract-version", "1")
                .bodySerialized(data)
                .sendSerialized(XboxLiveAuthResponse.class);
    }

    public static LoginResponse loginToMinecraft(String xstsToken) throws IOException, URISyntaxException {
        Map<Object, Object> data = new HashMap<>();
        data.put("xtoken", xstsToken);
        data.put("platform", "PC_LAUNCHER");

        return HttpUtils.post(MICROSOFT_MINECRAFT_LOGIN_URL)
                .bodySerialized(data)
                .sendSerialized(LoginResponse.class);
    }

    public static Entitlements getEntitlements(String accessToken) throws IOException, URISyntaxException {
        return HttpUtils.get(MICROSOFT_MINECRAFT_ENTITLEMENTS_URL + UUID.randomUUID()).bearer(accessToken).sendSerialized(Entitlements.class);
    }

    public static Profile getMcProfile(String accessToken) throws IOException, URISyntaxException {
        return HttpUtils.get(MICROSOFT_MINECRAFT_PROFILE_URL).bearer(accessToken).sendSerialized(Profile.class);
    }
}
